/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
//
// Copyright (c) 2010 INRIA
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as
// published by the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Author: Rao Naveed Bin Rais <Naveed-Bin.Rais@inria.fr>
//

#include "seq-header.h"
#include "ns3/address-utils.h"

namespace ns3 {

SeqNumHeader::SeqNumHeader ()
  : m_seqNum (0)
{}

SeqNumHeader::~SeqNumHeader ()
{}

TypeId 
SeqNumHeader::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::SeqNumHeader")
    .SetParent<Header> ()
    .AddConstructor<SeqNumHeader> ()
    ;
  return tid;
}

TypeId 
SeqNumHeader::GetInstanceTypeId (void) const
{
  return GetTypeId ();
}

void 
SeqNumHeader::Print (std::ostream &os) const
{
}

uint32_t 
SeqNumHeader::GetSerializedSize (void) const
{
  return 14;
}

void 
SeqNumHeader::Serialize (Buffer::Iterator i) const
{
  i.WriteU32 (m_seqNum);
  uint32_t temp = (uint32_t)m_sendTime.GetSeconds ();
  i.WriteU32 (temp);
  i.WriteU16 (m_priority);
  i.WriteU16 (m_numOfCopies);
  i.WriteU16 (m_bbCopyGiven);
}

uint32_t
SeqNumHeader::Deserialize (Buffer::Iterator start)
{
  Buffer::Iterator i = start;
  m_seqNum = i.ReadU32 ();
  m_sendTime = Seconds (i.ReadU32 ());
  m_priority = i.ReadU16 ();
  m_numOfCopies = i.ReadU16 ();
  m_bbCopyGiven = i.ReadU16 ();

  return i.GetDistanceFrom (start);
}
} // ns3 namespace

