/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
//
// Copyright (c) 2010 INRIA
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as
// published by the Free Software Foundation;
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// Author: Rao Naveed Bin Rais <Naveed-Bin.Rais@inria.fr>
//

#ifndef SEQ_HEADER_H
#define SEQ_HEADER_H

#include <map>
#include <list>
#include "ns3/header.h"
#include "ns3/mac48-address.h"
#include "ns3/packet.h"
#include "ns3/llc-snap-header.h"
#include "ns3/simulator.h"

#pragma once

namespace ns3 {

class SeqNumHeader : public Header 
{
public:

  SeqNumHeader ();
  ~SeqNumHeader ();

  static TypeId GetTypeId (void);
  virtual TypeId GetInstanceTypeId (void) const;
  virtual void Print (std::ostream &os) const;
  virtual uint32_t GetSerializedSize (void) const;
  virtual void Serialize (Buffer::Iterator start) const;
  virtual uint32_t Deserialize (Buffer::Iterator start);
  
  int GetSeqNum (void) { return m_seqNum; }
  void SetSeqNum (int seqNum) { m_seqNum = seqNum; }
  Time GetSendTime (void) { return m_sendTime; }
  void SetSendTime (Time time) { m_sendTime = time; }
  void SetPriority (bool priority) { m_priority = priority; }
  bool GetPriority (void) { return m_priority; }
  void SetNumOfCopies (uint16_t copies) { m_numOfCopies = copies; }
  uint16_t GetNumOfCopies (void) { return m_numOfCopies; }
  void SetCopyGiven (bool bbCopyGiven) { m_bbCopyGiven = bbCopyGiven; }
  bool GetCopyGiven (void) { return m_bbCopyGiven; }
  

private:
  uint32_t  m_seqNum;
  Time m_sendTime;
  bool m_priority;
  uint16_t m_numOfCopies;
  bool m_bbCopyGiven;
};

} // ns3 namespace

#endif

